"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.StageTrace = exports.addNexusMessageTrace = exports.sampleMessages = void 0;
const common_utils_1 = require("@fluidframework/common-utils");
const server_services_client_1 = require("@fluidframework/server-services-client");
const server_services_core_1 = require("@fluidframework/server-services-core");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
/**
 * Whether to add a trace to a message based on sampling rate.
 */
function sampleMessages(numberOfMessagesPerTrace) {
    return (0, server_services_client_1.getRandomInt)(numberOfMessagesPerTrace) === 0;
}
exports.sampleMessages = sampleMessages;
/**
 * Add a trace to a message depending on sampling rate.
 */
function addNexusMessageTrace(message, numberOfMessagesPerTrace, clientId, tenantId, documentId) {
    if (message &&
        server_services_core_1.DefaultServiceConfiguration.enableTraces &&
        sampleMessages(numberOfMessagesPerTrace)) {
        if (message.traces === undefined) {
            message.traces = [];
        }
        message.traces.push({
            action: "start",
            service: "nexus",
            timestamp: Date.now(),
        });
        const lumberjackProperties = {
            [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: tenantId,
            [server_services_telemetry_1.BaseTelemetryProperties.documentId]: documentId,
            clientId,
            clientSequenceNumber: message.clientSequenceNumber,
            traces: message.traces,
            opType: message.type,
        };
        server_services_telemetry_1.Lumberjack.info(`Message received by Nexus.`, lumberjackProperties);
    }
    return message;
}
exports.addNexusMessageTrace = addNexusMessageTrace;
class StageTrace {
    constructor(initialStage) {
        this.traces = [];
        this.lastStampedTraceTime = common_utils_1.performance.now();
        if (initialStage) {
            this.traces.push({ stage: initialStage.toString(), ts: 0 });
        }
    }
    get trace() {
        return this.traces;
    }
    stampStage(stage) {
        const now = common_utils_1.performance.now();
        this.traces.push({ stage: stage.toString(), ts: now - this.lastStampedTraceTime });
        this.lastStampedTraceTime = now;
    }
}
exports.StageTrace = StageTrace;
//# sourceMappingURL=trace.js.map